<?php
/***************************************************************************
 *                                                                          *
 *   (c) 2004 Vladimir V. Kalynyak, Alexey V. Vinokurov, Ilya M. Shalnev    *
 *                                                                          *
 * This  is  commercial  software,  only  users  who have purchased a valid *
 * license  and  accept  to the terms of the  License Agreement can install *
 * and use this program.                                                    *
 *                                                                          *
 ****************************************************************************
 * PLEASE READ THE FULL TEXT  OF THE SOFTWARE  LICENSE   AGREEMENT  IN  THE *
 * "copyright.txt" FILE PROVIDED WITH THIS DISTRIBUTION PACKAGE.            *
 ****************************************************************************/

namespace Tygh\Shippings\Services;

use Boxberry\Client\LocationFinder;
use Boxberry\Collections\ListCitiesCollection;
use Boxberry\Models\City;
use Boxberry\Models\DeliveryCalculation;
use Boxberry\Models\DeliveryCosts;
use Boxberry\Models\Point;
use Tygh\Payments\Processors\YandexMoneyMWS\Exception;
use Tygh\Shippings\IService;
use Tygh\Registry;
use Boxberry\Client\Client;
use Tygh\Tygh;

class Boxberry implements IService
{
    protected $client = null;
    protected $weight = null;
    protected $shipping_info;
    private $location;
    private $prepaid = false;

    /**
     * Boxberry constructor.
     */
    public function __construct()
    {
        $this->client = new Client();
        $this->location = new LocationFinder();
    }

    public function prepareData($shipping_info)
    {

        $group_key = $shipping_info['keys']['group_key'];
        $shipping_id = $shipping_info['keys']['shipping_id'];
        $key = trim($shipping_info['service_params']['password']);

        //$boxberry_target_start = $shipping_info['service_params']['boxberry_target_start'];

        $default_weight = ($shipping_info['service_params']['default_weight'] == "" || $shipping_info['service_params']['default_weight'] == "0") ? "1000" : $shipping_info['service_params']['default_weight'];
        $min_weight = $shipping_info['service_params']['min_weight'];
        $max_weight = $shipping_info['service_params']['max_weight'];
        $payment_sum = round($shipping_info['package_info']['C']);


        $this->client->setApiUrl($shipping_info['service_params']['api_url']);
        $this->client->setKey($key);
        $widgetKeyMethod = $this->client->getKeyIntegration();
        $widgetKeyMethod->setToken($key);
        $widgetResponse = $this->client->execute($widgetKeyMethod);
        $widget_key = $widgetResponse->getWidgetKey();

        $weight_data = fn_expand_weight($shipping_info['package_info']['W']);
        $weight = $weight_data['plain'] * Registry::get('settings.General.weight_symbol_grams') / 1000;
        $weight = sprintf('%.3f', round((double) $weight + 0.00000000001, 3)) * 1000;

        if (empty($weight) || ($weight === (float)1 && !empty($default_weight))){
            $weight = $default_weight;
        }

        if ($weight < $min_weight || $weight > $max_weight){
            return false;
        }

        $this->weight = $weight;

        $this->shipping_info = $shipping_info;

        Tygh::$app['session']['cart']['shippings_extra']['boxberry'][$group_key][$shipping_id]['apiKey'] = $key;
        Tygh::$app['session']['cart']['shippings_extra']['boxberry'][$group_key][$shipping_id]['apiKeyWidget'] = $widget_key;
        // Tygh::$app['session']['cart']['shippings_extra']['boxberry'][$group_key][$shipping_id]['boxberry_target_start'] = $boxberry_target_start;
        Tygh::$app['session']['cart']['shippings_extra']['boxberry'][$group_key][$shipping_id]['boxberry_weight'] = (float)$weight;
        Tygh::$app['session']['cart']['shippings_extra']['boxberry'][$group_key][$shipping_id]['boxberry_ordersum'] = $payment_sum;
        Tygh::$app['session']['cart']['shippings_extra']['boxberry'][$group_key][$shipping_id]['widget_url'] = $shipping_info['service_params']['widget_url'];
        Tygh::$app['session']['cart']['shippings_extra']['boxberry'][$group_key][$shipping_id]['api_url'] = $shipping_info['service_params']['api_url'];
        Tygh::$app['session']['cart']['shippings_extra']['boxberry'][$group_key][$shipping_id]['sucrh'] = $shipping_info['service_params']['sucrh'];
        if ($shipping_info['service_code'] == 'boxberry_courier_prepaid' || $shipping_info['service_code'] == 'boxberry_self_prepaid') {
            Tygh::$app['session']['cart']['shippings_extra']['boxberry'][$group_key][$shipping_id]['boxberry_paymentsum'] = 0;
        } else {
            Tygh::$app['session']['cart']['shippings_extra']['boxberry'][$group_key][$shipping_id]['boxberry_paymentsum'] = $payment_sum;
        }
    }

    public function processResponse($response)
    {
        if (isset($this->shipping_info['service_params']['margin_percent']) && is_numeric($this->shipping_info['service_params']['margin_percent'])) {
            $response['cost'] += ($response['cost'] * $this->shipping_info['service_params']['margin_percent']) / 100;
        }
        if (isset($this->shipping_info['service_params']['margin_plus']) && is_numeric($this->shipping_info['service_params']['margin_plus'])) {
            $response['cost'] += $this->shipping_info['service_params']['margin_plus'];
        }

        return $response;
    }

    public function processErrors($response)
    {

        if (!is_array($response) || (isset($response['error']) && $response['error'])) {
            return $response['error'];
        }
        return false;
    }

    public function allowMultithreading()
    {
        return false;
    }

    public function getRequestData()
    {

        return array(
            'method' => 'GET',
            'url' => $this->client->getApiUrl(),
            'data' => array(),
        );
    }

    public function getSimpleRates()
    {
        $arError = [
            'cost' => false,
            'error' => false,
            'delivery_time' => false,
        ];

        if (strpos($this->shipping_info['service_code'], 'prepaid') !== false) {
            $this->prepaid = true;
        }

        if (!$this->prepaid && !$this->isCodAvailableForCountry($this->shipping_info['package_info']['location']['country'])) {
            return $arError;
        }

        $selectedCity = mb_strtoupper($this->shipping_info['package_info']['location']['city']);
        $selectedState = mb_strtoupper($this->shipping_info['package_info']['location']['state_descr']);

        if (empty($selectedCity)) {
            return $arError;
        }

        if (!$this->location->getCityCode()) {
            try {
                $listCitiesFull = $this->client->getListCitiesFull();
                $listCitiesFullResponse = $this->client->execute($listCitiesFull);
            } catch (\Exception $e) {
                return $arError;
            }

            /** @var ListCitiesCollection $listCitiesFullResponse */
            $this->location->setCities($listCitiesFullResponse);

            if (!empty($selectedState)) {
                $this->location->setUserCityName($selectedCity);
                $this->location->setUserRegionName($selectedState);
                $this->location->findCodeByFullData();
            }

            if (!$this->location->getCityCode()) {
                $this->location->setUserCityName($selectedCity);
                $this->location->findCodeByCityName();
            }
        }

        if (!$this->location->getCityCode()) {
            return $arError;
        }

        $items = $this->shipping_info['package_info']['packages'];
        $itemsVolume = NULL;
        $fullPackage = array(
            'length' => 0,
            'width' => 0,
            'height' => 0
        );

        foreach ($items as $item) {
            if ($item['shipping_params']['box_length'] > 120 || $item['shipping_params']['box_width'] > 120 || $item['shipping_params']['box_height'] > 120) {
                $fullPackage['length'] += is_null($item['shipping_params']['box_length']) || $item['shipping_params']['box_length'] < 1 ? 0 : $item['shipping_params']['box_length'];
                $fullPackage['width'] += is_null($item['shipping_params']['box_width']) || $item['shipping_params']['box_width'] < 1 ? 0 : $item['shipping_params']['box_width'];
                $fullPackage['height'] += is_null($item['shipping_params']['box_height']) || $item['shipping_params']['box_height'] < 1 ? 0 : $item['shipping_params']['box_height'];
            } else {
                $itemsVolume += $item['shipping_params']['box_length'] * $item['shipping_params']['box_width'] * $item['shipping_params']['box_height'];
            }
        }

        if (!is_null($itemsVolume)) {
            $sideDimension = ceil($itemsVolume ** (1 / 3));
            $fullPackage['length'] += $sideDimension;
            $fullPackage['width'] += $sideDimension;
            $fullPackage['height'] += $sideDimension;
        }

        if ($fullPackage['length'] + $fullPackage['width'] + $fullPackage['height'] > 250 ||
            $fullPackage['length'] > 120 || $fullPackage['width'] > 120 || $fullPackage['height'] > 120) {
            return $arError;
        }

        $SurchagesSettingsMethod = $this->client->getWidgetSettings();
        $SurchagesSettings = $this->client->execute($SurchagesSettingsMethod);
        $blockedCityCodes = $SurchagesSettings->getCityCode();
        $getHideDeliveryDays = $SurchagesSettings->getHide_delivery_day();

        if (in_array($this->location->getCityCode(), $blockedCityCodes, true)) {
            return $arError;
        }

        $deliveryCalculation = $this->client->getDeliveryCalculation();

        if (strpos($this->shipping_info['service_code'], 'boxberry_courier') !== false) {
            $deliveryCalculation->setDeliveryType(DeliveryCalculation::COURIER_DELIVERY_TYPE_ID);
        } else {
            $deliveryCalculation->setDeliveryType(DeliveryCalculation::PICKUP_DELIVERY_TYPE_ID);
        }

        $deliveryCalculation->setRecipientCityId($this->location->getCityCode());

        if (!$this->prepaid) {
            $deliveryCalculation->setPaysum(round($this->shipping_info['package_info']['C']));
        } else {
            $deliveryCalculation->setPaySum(0);
        }

        $zip = '';

        if (isset($this->shipping_info['package_info']['location']['zipcode']) && !empty(trim($this->shipping_info['package_info']['location']['zipcode'])) && strlen($this->shipping_info['package_info']['location']['zipcode']) === 6 && is_numeric($this->shipping_info['package_info']['location']['zipcode'])) {
            $zip = $this->getZipCheck($this->shipping_info['package_info']['location']['zipcode']);
        }

        $deliveryCalculation->setZip($zip);
        $deliveryCalculation->setWeight($this->weight);
        $deliveryCalculation->setWidth($fullPackage['width']);
        $deliveryCalculation->setHeight($fullPackage['height']);
        $deliveryCalculation->setDepth($fullPackage['length']);
        $deliveryCalculation->setBoxSizes();
        $deliveryCalculation->setOrdersum(round($this->shipping_info['package_info']['C']));
        $deliveryCalculation->setCmsName('cscart');
        $deliveryCalculation->setUseShopSettings($this->shipping_info['service_params']['sucrh']);
        $deliveryCalculation->setUrl($_SERVER['HTTP_HOST']);
        $deliveryCalculation->setVersion('1.5');

        try {
            /** @var DeliveryCalculation $responseObject */
            $responseObject = $this->client->execute($deliveryCalculation);

            if ($deliveryCalculation->getDeliveryType() == DeliveryCalculation::PICKUP_DELIVERY_TYPE_ID && $responseObject->getPriceBasePickup()) {
                $cost = fn_format_price_by_currency(round($responseObject->getTotalPricePickup()), 'RUB', CART_PRIMARY_CURRENCY);
                $deliveryPeriod = $responseObject->getDeliveryPeriodPickup();
            } elseif ($deliveryCalculation->getDeliveryType() == DeliveryCalculation::COURIER_DELIVERY_TYPE_ID && $responseObject->getPriceBaseCourier()) {
                $cost = fn_format_price_by_currency(round($responseObject->getTotalPriceCourier()), 'RUB', CART_PRIMARY_CURRENCY);
                $deliveryPeriod = $responseObject->getDeliveryPeriodCourier();
            } else {
                return $arError;
            }

            $deliveryHtml = false;

            if (isset($getHideDeliveryDays) && $getHideDeliveryDays === 0) {
                $deliveryHtml = $deliveryPeriod . ' ' . $this->plural($deliveryPeriod, 'рабочий день', 'рабочих дня', 'рабочих дней');
            }

            $response = array(
                'cost' => $cost,
                'delivery_time' => $deliveryHtml,
                'error' => null,
            );

        } catch (\Exception $e) {
            $response = $arError;
        }

        return $response;
    }

    private function getZipCheck($zip)
    {
        try {
            $zipCheck = $this->client->getZipCheck();
            $zipCheck->setZip($zip);
            $zipCheckResponse = $this->client->execute($zipCheck);

            if (!empty($zipCheckResponse) && $zipCheckResponse->getExpressDelivery() === 1) {
                return $zip;
            }
        } catch (\Exception $e) {
        }

        return '';
    }

    private function  plural($n, $form1, $form2, $form3) {
        $n  = abs($n) % 100;
        $n1 = $n % 10;

        if ($n > 10 && $n < 20) return $form3;
        if ($n1 > 1 && $n1 < 5) return $form2;
        if ($n1 == 1) return $form1;

        return $form3;
    }

    /**
     * @return bool
     */
    public function isPrepaid()
    {
        return $this->prepaid;
    }

    /**
     * @param bool $prepaid
     */
    public function setPrepaid($prepaid)
    {
        $this->prepaid = $prepaid;
    }

    private function isCodAvailableForCountry($country)
    {
        if ($country === 'RU' || $country === 'KZ') {
            return true;
        }

        return false;
    }

}
